--[[============================================================================
main.lua
============================================================================]]--

-- Placeholder for the dialog
local dialog = nil

-- Placeholder to expose the ViewBuilder outside the show_dialog() function
local vb = nil

-- Reload the script whenever this file is saved. 
-- Additionally, execute the attached function.
_AUTO_RELOAD_DEBUG = function()
  
end

-- Read from the manifest.xml file.
class "RenoiseScriptingTool" (renoise.Document.DocumentNode)
  function RenoiseScriptingTool:__init()    
    renoise.Document.DocumentNode.__init(self) 
    self:add_property("Name", "Untitled Tool")
    self:add_property("Id", "Unknown Id")
  end

local manifest = RenoiseScriptingTool()
local ok,err = manifest:load_from("manifest.xml")
local tool_name = manifest:property("Name").value
local tool_id = manifest:property("Id").value


--------------------------------------------------------------------------------
-- Main functions
--------------------------------------------------------------------------------

local function set_loop_to_selection(toggle_play_boolean)

  ----------------
  --GATHER DATA
  --Get selection data
  local rs = renoise.song()
  local se = rs.selection_in_pattern
  --[[
  where relevant data is
  loop start line = se.start_line
  loop end line= se.end_line
  --]]
  local cur_loop_start_seq =  rs.transport.loop_range[1].sequence
  local cur_loop_end_seq =  rs.transport.loop_range[2].sequence
  local cur_loop_start_line =  rs.transport.loop_range[1].line
  local cur_loop_end_line =  rs.transport.loop_range[2].line

  ----------------
  --HELPERS
  
  --Set loop action
  local function set_loop()
    --Init songpos object dummies
    local start_pos = rs.transport.edit_pos
    local end_pos = rs.transport.edit_pos
    --Make them right
    start_pos.line = se.start_line
    end_pos.line = se.end_line + 1
    --Set
    rs.transport.loop_range = {start_pos, end_pos}
    if toggle_play_boolean then
      --Play
      rs.transport.playback_pos   = start_pos
      rs.transport.playing = true
    end
  end
  
  --Clear loop action
  local function clear_loop()
    --Clear
    rs.transport.loop_block_enabled = false
    if toggle_play_boolean then
      --Stop
      rs.transport.playing = false
    end
  end
  
  --Check if loop is same as selection
  local function loop_range_is_selection_range()
    --does the loop span over sequences?
    local same_seq = cur_loop_start_seq == cur_loop_end_seq
    if not same_seq then
      return false
    end
    --do the lines match?
    local start_match = cur_loop_start_line == se.start_line
    local end_match = cur_loop_end_line == se.end_line + 1
    local lines_match = start_match and end_match
    if not lines_match then
      return false
    end
    --they match
    print("match")
    return true

  end

  ----------------
  --CHECKS
  --No selection -> nothing
  if not se then
    return
  end
  --Select action
  local action = nil
  local status_msg = nil
  if loop_range_is_selection_range() then 
    --Loop is same as selection -> clear loop
    action = clear_loop
    status_msg = "Loop cleared"
  else
    --Loop is different than selection -> set loop
    action = set_loop
    status_msg = "Loop set on selection"
  end
 
  ----------------
  --DO IT
  action()
  renoise.app():show_status(status_msg)
end





--------------------------------------------------------------------------------
-- Key Binding
--------------------------------------------------------------------------------


renoise.tool():add_keybinding {
  name = "Global:Transport:" .. tool_name.." + Play",
  invoke = function()
      set_loop_to_selection(true)
    end
}

renoise.tool():add_keybinding {
  name = "Global:Transport:" .. tool_name,
  invoke = function()
      set_loop_to_selection(false)
    end
}



--------------------------------------------------------------------------------
-- MIDI Mapping
--------------------------------------------------------------------------------

--[[
renoise.tool():add_midi_mapping {
  name = tool_id..":Show Dialog...",
  invoke = show_dialog
}
--]]
